@@ -18,7 +18,7 @@
 # See the LICENSE file for details.
 
 
-ALLOWED_PSYCOPG_VERSIONS = ('2.0.1', '2.0.2', '2.0.3', '2.0.4', '2.0.5')
+ALLOWED_PSYCOPG_VERSIONS = ('2.0.6',)
 
 import sys
 import time
@@ -97,8 +97,6 @@ class Connection(Shared.DC.ZRDB.Connection.Connection):
         self.zdatetime = zdatetime
         self.tilevel = tilevel
         self.encoding = encoding
-
-        self.set_type_casts()
         
         if check: self.connect(self.connection_string)
 
@@ -125,30 +123,23 @@ class Connection(Shared.DC.ZRDB.Connection.Connection):
             raise ImportError("psycopg version mismatch (imported %s)" %
                               psycopg2.__version__)
 
-        self.set_type_casts()
         self._v_connected = ''
         dbf = self.factory()
         
         # TODO: let the psycopg exception propagate, or not?
         self._v_database_connection = dbf(
-            self.connection_string, self.tilevel, self.encoding)
+            self.connection_string, self.tilevel, self.get_type_casts(), self.encoding)
         self._v_database_connection.open()
         self._v_connected = DateTime()
 
         return self
 
-    def set_type_casts(self):
+    def get_type_casts(self):
         # note that in both cases order *is* important
         if self.zdatetime:
-            # use zope internal datetime routines
-            register_type(ZDATETIME)
-            register_type(ZDATE)
-            register_type(ZTIME)
+            return ZDATETIME, ZDATE, ZTIME
         else:
-            # use the standard
-            register_type(DATETIME)
-            register_type(DATE)
-            register_type(TIME)
+            return DATETIME, DATE, TIME
 
     ## browsing and table/column management ##
 
@@ -220,29 +211,28 @@ for icon in ('table', 'view', 'stable', 'what', 'field', 'text', 'bin',
 # convert an ISO timestamp string from postgres to a Zope DateTime object
 def _cast_DateTime(iso, curs):
     if iso:
-        return DateTime(re.split("GMT\+?|GMT-?", iso)[0])
-	
-    # this will split us into [date, time, GMT/AM/PM(if there)]
-    #    dt = str.split(' ')
-    #    if len(dt) > 1:
-    #        # we now should split out any timezone info
-    #        dt[1] = dt[1].split('-')[0]
-    #        dt[1] = dt[1].split('+')[0]
-    #        return DateTime(' '.join(dt[:2]))
-    #    else:
-    #        return DateTime(dt[0])
+        if iso in ['-infinity', 'infinity']:
+            return iso
+        else:
+            return DateTime(iso)
 
 # convert an ISO date string from postgres to a Zope DateTime object
 def _cast_Date(iso, curs):
     if iso:
-        return DateTime(iso)
+        if iso in ['-infinity', 'infinity']:
+            return iso
+        else:
+            return DateTime(iso)
 
 # Convert a time string from postgres to a Zope DateTime object.
 # NOTE: we set the day as today before feeding to DateTime so
 # that it has the same DST settings.
 def _cast_Time(iso, curs):
     if iso:
-        return DateTime(time.strftime('%Y-%m-%d %H:%M:%S',
+        if iso in ['-infinity', 'infinity']:
+            return iso
+        else:
+            return DateTime(time.strftime('%Y-%m-%d %H:%M:%S',
                                       time.localtime(time.time())[:3]+
                                       time.strptime(iso[:8], "%H:%M:%S")[3:]))
 
