@@ -22,6 +22,7 @@
 #ifndef PSYCOPG_ASIS_H
 #define PSYCOPG_ASIS_H 1
 
+#define PY_SSIZE_T_CLEAN
 #include <Python.h>
 
 #ifdef __cplusplus
@@ -31,7 +32,7 @@ extern "C" {
 extern PyTypeObject asisType;
 
 typedef struct {
-    PyObject HEAD;
+    PyObject_HEAD
 
     /* this is the real object we wrap */
     PyObject *wrapped;
@@ -39,7 +40,7 @@ typedef struct {
 } asisObject;
 
 /* functions exported to psycopgmodule.c */
-    
+
 extern PyObject *psyco_AsIs(PyObject *module, PyObject *args);
 #define psyco_AsIs_doc \
     "AsIs(obj) -> new AsIs wrapper object"
