@@ -19,6 +19,7 @@
  * Foundation, 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 
+#define PY_SSIZE_T_CLEAN
 #include <Python.h>
 
 #define PSYCOPG_MODULE
@@ -88,8 +89,8 @@ PyObject *decimalType = NULL;
 ":return: New database connection\n"                                         \
 ":rtype: `extensions.connection`"
 
-static int
-_psyco_connect_fill_dsn(char *dsn, char *kw, char *v, int i)
+static size_t
+_psyco_connect_fill_dsn(char *dsn, char *kw, char *v, size_t i)
 {
     strcpy(&dsn[i], kw); i += strlen(kw);
     strcpy(&dsn[i], v); i += strlen(v);
@@ -125,42 +126,47 @@ _psyco_connect_fill_exc(connectionObject *conn)
 static PyObject *
 psyco_connect(PyObject *self, PyObject *args, PyObject *keywds)
 {
-    PyObject *conn, *factory = NULL;
+    PyObject *conn = NULL, *factory = NULL;
     PyObject *pyport = NULL;
-    
-    int idsn=-1, iport=-1;
-    char *dsn=NULL, *database=NULL, *user=NULL, *password=NULL;
+
+    size_t idsn=-1;
+    int iport=-1;
+    char *dsn_static=NULL, *dsn_dynamic=NULL;
+    char *database=NULL, *user=NULL, *password=NULL;
     char *host=NULL, *sslmode=NULL;
     char port[16];
-    
+
     static char *kwlist[] = {"dsn", "database", "host", "port",
                              "user", "password", "sslmode",
                              "connection_factory", NULL};
-    
+
     if (!PyArg_ParseTupleAndKeywords(args, keywds, "|sssOsssO", kwlist,
-                                     &dsn, &database, &host, &pyport,
+                                     &dsn_static, &database, &host, &pyport,
                                      &user, &password, &sslmode, &factory)) {
         return NULL;
     }
 
     if (pyport && PyString_Check(pyport)) {
-	PyObject *pyint = PyInt_FromString(PyString_AsString(pyport), NULL, 10);
-	if (!pyint) return NULL;
-	iport = PyInt_AsLong(pyint);
+      PyObject *pyint = PyInt_FromString(PyString_AsString(pyport), NULL, 10);
+      if (!pyint) goto fail;
+      /* Must use PyInt_AsLong rather than PyInt_AS_LONG, because
+       * PyInt_FromString can return a PyLongObject: */
+      iport = PyInt_AsLong(pyint);
+      Py_DECREF(pyint);
     }
     else if (pyport && PyInt_Check(pyport)) {
-	iport = PyInt_AsLong(pyport);
+      iport = PyInt_AsLong(pyport);
     }
     else if (pyport != NULL) {
-	PyErr_SetString(PyExc_TypeError, "port must be a string or int");
-	return NULL;
+      PyErr_SetString(PyExc_TypeError, "port must be a string or int");
+      goto fail;
     }
 
     if (iport > 0)
       PyOS_snprintf(port, 16, "%d", iport);
 
-    if (dsn == NULL) {
-        int l = 45;  /* len("dbname= user= password= host= port= sslmode=\0") */
+    if (dsn_static == NULL) {
+        size_t l = 45; /* len("dbname= user= password= host= port= sslmode=\0") */
 
         if (database) l += strlen(database);
         if (host) l += strlen(host);
@@ -168,45 +174,60 @@ psyco_connect(PyObject *self, PyObject *args, PyObject *keywds)
         if (user) l += strlen(user);
         if (password) l += strlen(password);
         if (sslmode) l += strlen(sslmode);
-        
-        dsn = malloc(l*sizeof(char));
-        if (dsn == NULL) {
+
+        dsn_dynamic = malloc(l*sizeof(char));
+        if (dsn_dynamic == NULL) {
             PyErr_SetString(InterfaceError, "dynamic dsn allocation failed");
-            return NULL;
+            goto fail;
         }
 
         idsn = 0;
         if (database)
-            idsn = _psyco_connect_fill_dsn(dsn, " dbname=", database, idsn);
+            idsn = _psyco_connect_fill_dsn(dsn_dynamic, " dbname=", database, idsn);
         if (host)
-            idsn = _psyco_connect_fill_dsn(dsn, " host=", host, idsn);
+            idsn = _psyco_connect_fill_dsn(dsn_dynamic, " host=", host, idsn);
         if (iport > 0)
-            idsn = _psyco_connect_fill_dsn(dsn, " port=", port, idsn);
+            idsn = _psyco_connect_fill_dsn(dsn_dynamic, " port=", port, idsn);
         if (user)
-            idsn = _psyco_connect_fill_dsn(dsn, " user=", user, idsn);
+            idsn = _psyco_connect_fill_dsn(dsn_dynamic, " user=", user, idsn);
         if (password)
-            idsn = _psyco_connect_fill_dsn(dsn, " password=", password, idsn);
+            idsn = _psyco_connect_fill_dsn(dsn_dynamic, " password=", password, idsn);
         if (sslmode)
-            idsn = _psyco_connect_fill_dsn(dsn, " sslmode=", sslmode, idsn);
-        
+            idsn = _psyco_connect_fill_dsn(dsn_dynamic, " sslmode=", sslmode, idsn);
+
         if (idsn > 0) {
-            dsn[idsn] = '\0';
-            memmove(dsn, &dsn[1], idsn);
+            dsn_dynamic[idsn] = '\0';
+            memmove(dsn_dynamic, &dsn_dynamic[1], idsn);
         }
         else {
-            free(dsn);
             PyErr_SetString(InterfaceError, "missing dsn and no parameters");
-            return NULL;
+            goto fail;
         }
     }
 
-    Dprintf("psyco_connect: dsn = '%s'", dsn);
+    {
+      const char *dsn = (dsn_static != NULL ? dsn_static : dsn_dynamic);
+      Dprintf("psyco_connect: dsn = '%s'", dsn);
+
+      /* allocate connection, fill with errors and return it */
+      if (factory == NULL) factory = (PyObject *)&connectionType;
+      conn = PyObject_CallFunction(factory, "s", dsn);
+      if (conn) _psyco_connect_fill_exc((connectionObject*)conn);
+    }
+
+    goto cleanup;
+    fail:
+      assert (PyErr_Occurred());
+      if (conn != NULL) {
+        Py_DECREF(conn);
+        conn = NULL;
+      }
+      /* Fall through to cleanup: */
+    cleanup:
+      if (dsn_dynamic != NULL) {
+        free(dsn_dynamic);
+      }
 
-    /* allocate connection, fill with errors and return it */
-    if (factory == NULL) factory = (PyObject *)&connectionType;
-    conn = PyObject_CallFunction(factory, "s", dsn);
-    if (conn) _psyco_connect_fill_exc((connectionObject*)conn);
-    
     return conn;
 }
 
@@ -228,17 +249,40 @@ psyco_connect(PyObject *self, PyObject *args, PyObject *keywds)
 "    the string representation returned by PostgreSQL (`None` if ``NULL``)\n" \
 "    and ``cur`` is the cursor from which data are read."
 
+static void
+_psyco_register_type_set(PyObject **dict, PyObject *type)
+{
+    if (*dict == NULL)
+        *dict = PyDict_New();
+    typecast_add(type, *dict, 0);
+}
+
 static PyObject *
 psyco_register_type(PyObject *self, PyObject *args)
 {
-    PyObject *type;
+    PyObject *type, *obj = NULL;
 
-    if (!PyArg_ParseTuple(args, "O!", &typecastType, &type)) {
+    if (!PyArg_ParseTuple(args, "O!|O", &typecastType, &type, &obj)) {
         return NULL;
     }
 
-    typecast_add(type, 0);
-    
+    if (obj != NULL) {
+        if (obj->ob_type == &cursorType) {
+            _psyco_register_type_set(&(((cursorObject*)obj)->string_types), type);
+        }
+        else if (obj->ob_type == &connectionType) {
+            typecast_add(type, ((connectionObject*)obj)->string_types, 0);
+        }
+        else {
+            PyErr_SetString(PyExc_TypeError,
+                "argument 2 must be a connection, cursor or None");
+            return NULL;
+        }
+    }
+    else {
+        typecast_add(type, NULL, 0);
+    }
+
     Py_INCREF(Py_None);
     return Py_None;
 }
@@ -250,16 +294,16 @@ static void
 psyco_adapters_init(PyObject *mod)
 {
     PyObject *call;
-    
+
     microprotocols_add(&PyFloat_Type, NULL, (PyObject*)&asisType);
     microprotocols_add(&PyInt_Type, NULL, (PyObject*)&asisType);
     microprotocols_add(&PyLong_Type, NULL, (PyObject*)&asisType);
-    
+
     microprotocols_add(&PyString_Type, NULL, (PyObject*)&qstringType);
     microprotocols_add(&PyUnicode_Type, NULL, (PyObject*)&qstringType);
     microprotocols_add(&PyBuffer_Type, NULL, (PyObject*)&binaryType);
     microprotocols_add(&PyList_Type, NULL, (PyObject*)&listType);
-                       
+
 #ifdef HAVE_MXDATETIME
     /* the module has already been initialized, so we can obtain the callable
        objects directly from its dictionary :) */
@@ -280,7 +324,7 @@ psyco_adapters_init(PyObject *mod)
     call = PyMapping_GetItemString(mod, "IntervalFromPy");
     microprotocols_add((PyTypeObject*)pyDeltaTypeP, NULL, call);
 #endif
-    
+
 #ifdef HAVE_PYBOOL
     microprotocols_add(&PyBool_Type, NULL, (PyObject*)&pbooleanType);
 #endif
@@ -291,20 +335,81 @@ psyco_adapters_init(PyObject *mod)
 }
 
 /* psyco_encodings_fill
-   
+
    Fill the module's postgresql<->python encoding table */
 
 static encodingPair encodings[] = {
     {"SQL_ASCII",    "ascii"},
-    {"LATIN1",       "latin_1"},
-    {"UNICODE",      "utf_8"},
+    {"LATIN1",       "iso8859_1"},
+    {"LATIN2",       "iso8859_2"},
+    {"LATIN3",       "iso8859_3"},
+    {"LATIN4",       "iso8859_4"},
+    {"LATIN5",       "iso8859_9"},
+    {"LATIN6",       "iso8859_10"},
+    {"LATIN7",       "iso8859_13"},
+    {"LATIN8",       "iso8859_14"},
+    {"LATIN9",       "iso8859_15"},
+    {"ISO88591",     "iso8859_1"},
+    {"ISO88592",     "iso8859_2"},
+    {"ISO88593",     "iso8859_3"},
+    {"ISO88595",     "iso8859_5"},
+    {"ISO88596",     "iso8859_6"},
+    {"ISO88597",     "iso8859_7"},
+    {"ISO885913",    "iso8859_13"},
+    {"ISO88598",     "iso8859_8"},
+    {"ISO88599",     "iso8859_9"},
+    {"ISO885914",    "iso8859_14"},
+    {"ISO885915",    "iso8859_15"},
+    {"UNICODE",      "utf_8"}, /* Not valid in 8.2, backward compatibility */
     {"UTF8",         "utf_8"},
-    
-    /* some compatibility stuff */
-    {"LATIN-1",      "latin_1"},
-    
+    {"WIN950",       "cp950"},
+    {"Windows950",   "cp950"},
+    {"BIG5",         "big5"},
+    {"EUC_JP",       "euc_jp"},
+    {"EUC_KR",       "euc_kr"},
+    {"GB18030",      "gb18030"},
+    {"GBK",          "gbk"},
+    {"WIN936",       "gbk"},
+    {"Windows936",   "gbk"},
+    {"JOHAB",        "johab"},
+    {"KOI8",         "koi8_r"}, /* in PG: KOI8 == KOI8R == KOI8-R == KOI8-U
+                                   but in Python there is koi8_r AND koi8_u */
+    {"KOI8R",        "koi8_r"},
+    {"SJIS",         "cp932"},
+    {"Mskanji",      "cp932"},
+    {"ShiftJIS",     "cp932"},
+    {"WIN932",       "cp932"},
+    {"Windows932",   "cp932"},
+    {"UHC",          "cp949"},
+    {"WIN949",       "cp949"},
+    {"Windows949",   "cp949"},
+    {"WIN866",       "cp866"},
+    {"ALT",          "cp866"},
+    {"WIN874",       "cp874"},
+    {"WIN1250",      "cp1250"},
+    {"WIN1251",      "cp1251"},
+    {"WIN",          "cp1251"},
+    {"WIN1252",      "cp1252"},
+    {"WIN1253",      "cp1253"},
+    {"WIN1254",      "cp1254"},
+    {"WIN1255",      "cp1255"},
+    {"WIN1256",      "cp1256"},
+    {"WIN1257",      "cp1257"},
+    {"WIN1258",      "cp1258"},
+    {"ABC",          "cp1258"},
+    {"TCVN",         "cp1258"},
+    {"TCVN5712",     "cp1258"},
+    {"VSCII",        "cp1258"},
+
+/* those are missing from Python:                */
+/*    {"EUC_CN", "?"},                           */
+/*    {"EUC_TW", "?"},                           */
+/*    {"LATIN10", "?"},                          */
+/*    {"ISO885916", "?"},                        */
+/*    {"MULE_INTERNAL", "?"},                    */
     {NULL, NULL}
 };
+
 static void psyco_encodings_fill(PyObject *dict)
 {
     encodingPair *enc;
@@ -337,14 +442,14 @@ static struct {
     { "psycopg2.InterfaceError", &InterfaceError, &Error, InterfaceError_doc },
     { "psycopg2.DatabaseError", &DatabaseError, &Error, DatabaseError_doc },
     { "psycopg2.InternalError", &InternalError, &DatabaseError, InternalError_doc },
-    { "psycopg2.OperationalError", &OperationalError, &DatabaseError, 
+    { "psycopg2.OperationalError", &OperationalError, &DatabaseError,
         OperationalError_doc },
-    { "psycopg2.ProgrammingError", &ProgrammingError, &DatabaseError, 
+    { "psycopg2.ProgrammingError", &ProgrammingError, &DatabaseError,
         ProgrammingError_doc },
-    { "psycopg2.IntegrityError", &IntegrityError, &DatabaseError, 
+    { "psycopg2.IntegrityError", &IntegrityError, &DatabaseError,
         IntegrityError_doc },
     { "psycopg2.DataError", &DataError, &DatabaseError, DataError_doc },
-    { "psycopg2.NotSupportedError", &NotSupportedError, &DatabaseError, 
+    { "psycopg2.NotSupportedError", &NotSupportedError, &DatabaseError,
         NotSupportedError_doc },
     {NULL}  /* Sentinel */
 };
@@ -355,7 +460,7 @@ psyco_errors_init(void)
     /* the names of the exceptions here reflect the oranization of the
        psycopg2 module and not the fact the the original error objects
        live in _psycopg */
-       
+
     int i;
     PyObject *dict;
     PyObject *base;
@@ -390,7 +495,7 @@ psyco_errors_fill(PyObject *dict)
     PyDict_SetItemString(dict, "ProgrammingError", ProgrammingError);
     PyDict_SetItemString(dict, "IntegrityError", IntegrityError);
     PyDict_SetItemString(dict, "DataError", DataError);
-    PyDict_SetItemString(dict, "NotSupportedError", NotSupportedError);    
+    PyDict_SetItemString(dict, "NotSupportedError", NotSupportedError);
 }
 
 void
@@ -409,17 +514,17 @@ psyco_errors_set(PyObject *type)
 }
 
 /* psyco_error_new
-  
+
    Create a new error of the given type with extra attributes. */
-   
+
 void
-psyco_set_error(PyObject *exc, PyObject *curs, char *msg, 
+psyco_set_error(PyObject *exc, PyObject *curs, char *msg,
                  char *pgerror, char *pgcode)
 {
     PyObject *t;
-    
+
     PyObject *err = PyObject_CallFunction(exc, "s", msg);
-  
+
     if (err) {
         if (pgerror) {
             t = PyString_FromString(pgerror);
@@ -438,16 +543,16 @@ psyco_set_error(PyObject *exc, PyObject *curs, char *msg,
         }
         PyObject_SetAttrString(err, "pgcode", t);
         Py_DECREF(t);
-        
+
         if (curs)
             PyObject_SetAttrString(err, "cursor", curs);
         else
             PyObject_SetAttrString(err, "cursor", Py_None);
 
         PyErr_SetObject(exc, err);
-	Py_DECREF(err);
+    Py_DECREF(err);
     }
-} 
+}
 
 /* psyco_decimal_init
 
@@ -505,7 +610,7 @@ static PyMethodDef psycopgMethods[] = {
      METH_VARARGS, psyco_TimestampFromTicks_doc},
     {"List",  (PyCFunction)psyco_List,
      METH_VARARGS, psyco_List_doc},
-    
+
 #ifdef HAVE_MXDATETIME
     {"DateFromMx",  (PyCFunction)psyco_DateFromMx,
      METH_VARARGS, psyco_DateFromMx_doc},
@@ -514,7 +619,7 @@ static PyMethodDef psycopgMethods[] = {
     {"TimestampFromMx",  (PyCFunction)psyco_TimestampFromMx,
      METH_VARARGS, psyco_TimestampFromMx_doc},
     {"IntervalFromMx",  (PyCFunction)psyco_IntervalFromMx,
-     METH_VARARGS, psyco_IntervalFromMx_doc}, 
+     METH_VARARGS, psyco_IntervalFromMx_doc},
 #endif
 
 #ifdef HAVE_PYDATETIME
@@ -525,7 +630,7 @@ static PyMethodDef psycopgMethods[] = {
     {"TimestampFromPy",  (PyCFunction)psyco_TimestampFromPy,
      METH_VARARGS, psyco_TimestampFromPy_doc},
     {"IntervalFromPy",  (PyCFunction)psyco_IntervalFromPy,
-     METH_VARARGS, psyco_IntervalFromPy_doc},     
+     METH_VARARGS, psyco_IntervalFromPy_doc},
 #endif
 
     {NULL, NULL, 0, NULL}        /* Sentinel */
@@ -535,7 +640,7 @@ PyMODINIT_FUNC
 init_psycopg(void)
 {
     static void *PSYCOPG_API[PSYCOPG_API_pointers];
-    
+
     PyObject *module, *dict;
     PyObject *c_api_object;
 
@@ -551,7 +656,7 @@ init_psycopg(void)
     asisType.ob_type       = &PyType_Type;
     listType.ob_type       = &PyType_Type;
     chunkType.ob_type      = &PyType_Type;
-    
+
     if (PyType_Ready(&connectionType) == -1) return;
     if (PyType_Ready(&cursorType) == -1) return;
     if (PyType_Ready(&typecastType) == -1) return;
@@ -561,18 +666,18 @@ init_psycopg(void)
     if (PyType_Ready(&asisType) == -1) return;
     if (PyType_Ready(&listType) == -1) return;
     if (PyType_Ready(&chunkType) == -1) return;
-    
+
 #ifdef HAVE_PYBOOL
     pbooleanType.ob_type   = &PyType_Type;
     if (PyType_Ready(&pbooleanType) == -1) return;
 #endif
-    
+
     /* import mx.DateTime module, if necessary */
 #ifdef HAVE_MXDATETIME
     mxdatetimeType.ob_type = &PyType_Type;
     if (PyType_Ready(&mxdatetimeType) == -1) return;
     if (mxDateTime_ImportModuleAndAPI() != 0) {
-        Dprintf("initpsycopg: why marc hide mx.DateTime again?!"); 
+        Dprintf("initpsycopg: why marc hide mx.DateTime again?!");
         PyErr_SetString(PyExc_ImportError, "can't import mx.DateTime module");
         return;
     }
@@ -583,7 +688,7 @@ init_psycopg(void)
 #ifdef HAVE_PYDATETIME
     pyDateTimeModuleP = PyImport_ImportModule("datetime");
     if (pyDateTimeModuleP == NULL) {
-        Dprintf("initpsycopg: can't import datetime module"); 
+        Dprintf("initpsycopg: can't import datetime module");
         PyErr_SetString(PyExc_ImportError, "can't import datetime module");
         return;
     }
@@ -596,20 +701,20 @@ init_psycopg(void)
     pyTimeTypeP = PyObject_GetAttrString(pyDateTimeModuleP, "time");
     pyDateTimeTypeP = PyObject_GetAttrString(pyDateTimeModuleP, "datetime");
     pyDeltaTypeP = PyObject_GetAttrString(pyDateTimeModuleP, "timedelta");
-#endif    
+#endif
 
     /* import psycopg2.tz anyway (TODO: replace with C-level module?) */
     pyPsycopgTzModule = PyImport_ImportModule("psycopg2.tz");
     if (pyPsycopgTzModule == NULL) {
-        Dprintf("initpsycopg: can't import psycopg2.tz module"); 
+        Dprintf("initpsycopg: can't import psycopg2.tz module");
         PyErr_SetString(PyExc_ImportError, "can't import psycopg2.tz module");
-        return;        
+        return;
     }
-    pyPsycopgTzLOCAL = 
-        PyObject_GetAttrString(pyPsycopgTzModule, "LOCAL"); 
-    pyPsycopgTzFixedOffsetTimezone = 
+    pyPsycopgTzLOCAL =
+        PyObject_GetAttrString(pyPsycopgTzModule, "LOCAL");
+    pyPsycopgTzFixedOffsetTimezone =
         PyObject_GetAttrString(pyPsycopgTzModule, "FixedOffsetTimezone");
-        
+
     /* initialize the module and grab module's dictionary */
     module = Py_InitModule("_psycopg", psycopgMethods);
     dict = PyModule_GetDict(module);
@@ -626,33 +731,33 @@ init_psycopg(void)
     psycoEncodings = PyDict_New();
     psyco_encodings_fill(psycoEncodings);
     psyco_decimal_init();
-    
+
     /* set some module's parameters */
     PyModule_AddStringConstant(module, "__version__", PSYCOPG_VERSION);
     PyModule_AddStringConstant(module, "__doc__", "psycopg PostgreSQL driver");
     PyModule_AddObject(module, "apilevel", PyString_FromString(APILEVEL));
     PyModule_AddObject(module, "threadsafety", PyInt_FromLong(THREADSAFETY));
     PyModule_AddObject(module, "paramstyle", PyString_FromString(PARAMSTYLE));
-    
+
     /* put new types in module dictionary */
     PyModule_AddObject(module, "connection", (PyObject*)&connectionType);
     PyModule_AddObject(module, "cursor", (PyObject*)&cursorType);
     PyModule_AddObject(module, "ISQLQuote", (PyObject*)&isqlquoteType);
-    
+
     /* encodings dictionary in module dictionary */
     PyModule_AddObject(module, "encodings", psycoEncodings);
-    
+
     /* initialize default set of typecasters */
     typecast_init(dict);
 
     /* initialize microprotocols layer */
     microprotocols_init(dict);
     psyco_adapters_init(dict);
-    
+
     /* create a standard set of exceptions and add them to the module's dict */
     psyco_errors_init();
     psyco_errors_fill(dict);
-    
+
     /* Solve win32 build issue about non-constant initializer element */
     cursorType.tp_alloc = PyType_GenericAlloc;
     binaryType.tp_alloc = PyType_GenericAlloc;
@@ -663,10 +768,14 @@ init_psycopg(void)
     qstringType.tp_alloc = PyType_GenericAlloc;
     listType.tp_alloc = PyType_GenericAlloc;
     chunkType.tp_alloc = PyType_GenericAlloc;
-    
+
 #ifdef HAVE_PYDATETIME
     pydatetimeType.tp_alloc = PyType_GenericAlloc;
 #endif
-    
+
+#ifdef HAVE_MXDATETIME
+    mxdatetimeType.tp_alloc = PyType_GenericAlloc;
+#endif
+
     Dprintf("initpsycopg: module initialization complete");
 }
