@@ -19,6 +19,7 @@
  * Foundation, 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 
+#define PY_SSIZE_T_CLEAN
 #include <Python.h>
 #include <structmember.h>
 
@@ -39,7 +40,7 @@ skip_until_space(char *s)
 }
 
 static char *
-skip_until_space2(char *s, int *len)
+skip_until_space2(char *s, Py_ssize_t *len)
 {
     while (*len > 0 && *s && *s != ' ') {
         s++; (*len)--;
@@ -48,13 +49,14 @@ skip_until_space2(char *s, int *len)
 }
 
 static int
-typecast_parse_date(char* s, char** t, int* len,
+typecast_parse_date(char* s, char** t, Py_ssize_t* len,
                      int* year, int* month, int* day)
 {
     int acc = -1, cz = 0;
-    
-    Dprintf("typecast_parse_date: len = %d, s = %s", *len, s);
-     
+
+    Dprintf("typecast_parse_date: len = " FORMAT_CODE_PY_SSIZE_T ", s = %s",
+      *len, s);
+
     while (cz < 3 && *len > 0 && *s) {
         switch (*s) {
         case '-':
@@ -67,7 +69,7 @@ typecast_parse_date(char* s, char** t, int* len,
             break;
         default:
             acc = (acc == -1 ? 0 : acc*10) + ((int)*s - (int)'0');
-            break;            
+            break;
         }
 
         s++; (*len)--;
@@ -77,24 +79,25 @@ typecast_parse_date(char* s, char** t, int* len,
         *day = acc;
         cz += 1;
     }
-    if (t != NULL) *t = s;    
+    if (t != NULL) *t = s;
 
     return cz;
 }
 
 static int
-typecast_parse_time(char* s, char** t, int* len,
+typecast_parse_time(char* s, char** t, Py_ssize_t* len,
                      int* hh, int* mm, int* ss, int* us, int* tz)
 {
     int acc = -1, cz = 0;
     int tzs = 1, tzhh = 0, tzmm = 0;
     int usd = 0;
-    
+
     /* sets microseconds and timezone to 0 because they may be missing */
     *us = *tz = 0;
-    
-    Dprintf("typecast_parse_time: len = %d, s = %s", *len, s);
-     
+
+    Dprintf("typecast_parse_time: len = " FORMAT_CODE_PY_SSIZE_T ", s = %s",
+      *len, s);
+
     while (cz < 6 && *len > 0 && *s) {
         switch (*s) {
         case ':':
@@ -123,7 +126,7 @@ typecast_parse_time(char* s, char** t, int* len,
         default:
             acc = (acc == -1 ? 0 : acc*10) + ((int)*s - (int)'0');
             if (cz == 3) usd += 1;
-            break;            
+            break;
         }
 
         s++; (*len)--;
@@ -136,13 +139,13 @@ typecast_parse_time(char* s, char** t, int* len,
         else if (cz == 5)   tzmm = acc;
     }
     if (t != NULL) *t = s;
-    
+
     *tz = tzs * tzhh*60 + tzmm;
-    
-    if (*us != 0.0) {
-        while (usd++ < 6) *us *= 10.0;
+
+    if (*us != 0) {
+        while (usd++ < 6) *us *= 10;
     }
-    
+
     return cz;
 }
 
@@ -168,7 +171,7 @@ typecastObject_initlist typecast_pydatetime[] = {
     {"PYDATETIME", typecast_DATETIME_types, typecast_PYDATETIME_cast},
     {"PYTIME", typecast_TIME_types, typecast_PYTIME_cast},
     {"PYDATE", typecast_DATE_types, typecast_PYDATE_cast},
-    {"PYINTERVAL", typecast_INTERVAL_types, typecast_PYINTERVAL_cast},    
+    {"PYINTERVAL", typecast_INTERVAL_types, typecast_PYINTERVAL_cast},
     {NULL, NULL, NULL}
 };
 #endif
@@ -179,7 +182,7 @@ typecastObject_initlist typecast_mxdatetime[] = {
     {"MXDATETIME", typecast_DATETIME_types, typecast_MXDATE_cast},
     {"MXTIME", typecast_TIME_types, typecast_MXTIME_cast},
     {"MXDATE", typecast_DATE_types, typecast_MXDATE_cast},
-    {"MXINTERVAL", typecast_INTERVAL_types, typecast_MXINTERVAL_cast},    
+    {"MXINTERVAL", typecast_INTERVAL_types, typecast_MXINTERVAL_cast},
     {NULL, NULL, NULL}
 };
 #endif
@@ -203,20 +206,20 @@ int
 typecast_init(PyObject *dict)
 {
     int i;
-    
+
     /* create type dictionary and put it in module namespace */
     psyco_types = PyDict_New();
     psyco_binary_types = PyDict_New();
-    
+
     if (!psyco_types || !psyco_binary_types) {
         Py_XDECREF(psyco_types);
         Py_XDECREF(psyco_binary_types);
         return -1;
-    }                         
+    }
 
     PyDict_SetItemString(dict, "string_types", psyco_types);
     PyDict_SetItemString(dict, "binary_types", psyco_binary_types);
-    
+
     /* insert the cast types into the 'types' dictionary and register them in
        the module dictionary */
     for (i = 0; typecast_builtins[i].name != NULL; i++) {
@@ -226,7 +229,7 @@ typecast_init(PyObject *dict)
 
         t = (typecastObject *)typecast_from_c(&(typecast_builtins[i]), dict);
         if (t == NULL) return -1;
-        if (typecast_add((PyObject *)t, 0) != 0) return -1;
+        if (typecast_add((PyObject *)t, NULL, 0) != 0) return -1;
 
         PyDict_SetItem(dict, t->name, (PyObject *)t);
 
@@ -235,7 +238,7 @@ typecast_init(PyObject *dict)
             psyco_default_binary_cast = (PyObject *)t;
         }
     }
-    
+
     /* create and save a default cast object (but does not register it) */
     psyco_default_cast = typecast_from_c(&typecast_default, dict);
 
@@ -258,32 +261,32 @@ typecast_init(PyObject *dict)
         PyDict_SetItem(dict, t->name, (PyObject *)t);
     }
 #endif
-    
+
     return 0;
 }
 
 /* typecast_add - add a type object to the dictionary */
 int
-typecast_add(PyObject *obj, int binary)
+typecast_add(PyObject *obj, PyObject *dict, int binary)
 {
     PyObject *val;
-    int len, i;
+    Py_ssize_t len, i;
 
     typecastObject *type = (typecastObject *)obj;
-    
-    Dprintf("typecast_add: object at %p, values refcnt = %d",
-            obj, type->values->ob_refcnt);
+
+    Dprintf("typecast_add: object at %p, values refcnt = "
+        FORMAT_CODE_PY_SSIZE_T,
+        obj, type->values->ob_refcnt
+      );
+
+    if (dict == NULL)
+        dict = (binary ? psyco_binary_types : psyco_types);
 
     len = PyTuple_Size(type->values);
     for (i = 0; i < len; i++) {
         val = PyTuple_GetItem(type->values, i);
         Dprintf("typecast_add:     adding val: %ld", PyInt_AsLong(val));
-        if (binary) {
-            PyDict_SetItem(psyco_binary_types, val, obj);
-        }
-        else {
-            PyDict_SetItem(psyco_types, val, obj);
-        }
+        PyDict_SetItem(dict, val, obj);
     }
 
     Dprintf("typecast_add:     base caster: %p", type->bcast);
@@ -302,8 +305,9 @@ typecast_cmp(PyObject *obj1, PyObject* obj2)
     typecastObject *self = (typecastObject*)obj1;
     typecastObject *other = NULL;
     PyObject *number = NULL;
-    int i, j, res = -1;
-    
+    Py_ssize_t i, j;
+    int res = -1;
+
     if (PyObject_TypeCheck(obj2, &typecastType)) {
         other = (typecastObject*)obj2;
     }
@@ -312,25 +316,25 @@ typecast_cmp(PyObject *obj1, PyObject* obj2)
     }
 
     Dprintf("typecast_cmp: other = %p, number = %p", other, number);
-    
+
     for (i=0; i < PyObject_Length(self->values) && res == -1; i++) {
         long int val = PyInt_AsLong(PyTuple_GET_ITEM(self->values, i));
-        
+
         if (other != NULL) {
             for (j=0; j < PyObject_Length(other->values); j++) {
                 if (PyInt_AsLong(PyTuple_GET_ITEM(other->values, j)) == val) {
-                    res = 0; break;   
+                    res = 0; break;
                 }
             }
         }
-        
+
         else if (number != NULL) {
             if (PyInt_AsLong(number) == val) {
                 res = 0; break;
             }
         }
     }
-    
+
     Py_XDECREF(number);
     return res;
 }
@@ -340,29 +344,29 @@ typecast_richcompare(PyObject *obj1, PyObject* obj2, int opid)
 {
     PyObject *result = NULL;
     int res = typecast_cmp(obj1, obj2);
-    
+
     if (PyErr_Occurred()) return NULL;
-    
+
     if ((opid == Py_EQ && res == 0) || (opid != Py_EQ && res != 0))
         result = Py_True;
     else
         result = Py_False;
-    
+
     Py_INCREF(result);
     return result;
 }
-     
+
 static struct PyMemberDef typecastObject_members[] = {
     {"name", T_OBJECT, OFFSETOF(name), RO},
     {"values", T_OBJECT, OFFSETOF(values), RO},
     {NULL}
 };
-    
+
 static void
 typecast_dealloc(PyObject *obj)
 {
     typecastObject *self = (typecastObject*)obj;
-    
+
     Py_XDECREF(self->values);
     Py_XDECREF(self->name);
     Py_XDECREF(self->pcast);
@@ -372,9 +376,9 @@ typecast_dealloc(PyObject *obj)
 
 static PyObject *
 typecast_call(PyObject *obj, PyObject *args, PyObject *kwargs)
-{   
+{
     PyObject *string, *cursor;
-    
+
     if (!PyArg_ParseTuple(args, "OO", &string, &cursor)) {
         return NULL;
     }
@@ -390,9 +394,9 @@ PyTypeObject typecastType = {
     "psycopg2._psycopg.type",
     sizeof(typecastObject),
     0,
-    
+
     typecast_dealloc, /*tp_dealloc*/
-    0,          /*tp_print*/ 
+    0,          /*tp_print*/
     0,          /*tp_getattr*/
     0,          /*tp_setattr*/
     typecast_cmp, /*tp_compare*/
@@ -410,7 +414,7 @@ PyTypeObject typecastType = {
 
     Py_TPFLAGS_HAVE_RICHCOMPARE, /*tp_flags*/
     "psycopg type-casting object", /*tp_doc*/
-    
+
     0,          /*tp_traverse*/
     0,          /*tp_clear*/
 
@@ -427,11 +431,11 @@ PyTypeObject typecastType = {
     0,          /*tp_getset*/
     0,          /*tp_base*/
     0,          /*tp_dict*/
-    
+
     0,          /*tp_descr_get*/
     0,          /*tp_descr_set*/
     0,          /*tp_dictoffset*/
-    
+
     0, /*tp_init*/
     0, /*tp_alloc  will be set to PyType_GenericAlloc in module init*/
     0, /*tp_new*/
@@ -452,11 +456,12 @@ typecast_new(PyObject *name, PyObject *values, PyObject *cast, PyObject *base)
     obj = PyObject_NEW(typecastObject, &typecastType);
     if (obj == NULL) return NULL;
 
-    Dprintf("typecast_new: new type at = %p, refcnt = %d", obj, obj->ob_refcnt);
-             
+    Dprintf("typecast_new: new type at = %p, refcnt = " FORMAT_CODE_PY_SSIZE_T,
+      obj, obj->ob_refcnt);
+
     Py_INCREF(values);
     obj->values = values;
-    
+
     if (name) {
         Py_INCREF(name);
         obj->name = name;
@@ -477,26 +482,26 @@ typecast_new(PyObject *name, PyObject *values, PyObject *cast, PyObject *base)
         Py_INCREF(cast);
         obj->pcast = cast;
     }
-    
+
     Dprintf("typecast_new: typecast object created at %p", obj);
-    
+
     return (PyObject *)obj;
 }
 
 PyObject *
 typecast_from_python(PyObject *self, PyObject *args, PyObject *keywds)
 {
-    PyObject *v, *name, *cast = NULL, *base = NULL;
+    PyObject *v, *name = NULL, *cast = NULL, *base = NULL;
 
     static char *kwlist[] = {"values", "name", "castobj", "baseobj", NULL};
-    
-    if (!PyArg_ParseTupleAndKeywords(args, keywds, "O!|O!OO", kwlist, 
+
+    if (!PyArg_ParseTupleAndKeywords(args, keywds, "O!|O!OO", kwlist,
                                      &PyTuple_Type, &v,
                                      &PyString_Type, &name,
                                      &cast, &base)) {
         return NULL;
     }
-    
+
     return typecast_new(name, v, cast, base);
 }
 
@@ -505,7 +510,7 @@ typecast_from_c(typecastObject_initlist *type, PyObject *dict)
 {
     PyObject *tuple, *base = NULL;
     typecastObject *obj;
-    int i, len = 0;
+    Py_ssize_t i, len = 0;
 
     /* before doing anything else we look for the base */
     if (type->base) {
@@ -518,18 +523,18 @@ typecast_from_c(typecastObject_initlist *type, PyObject *dict)
     }
 
     while (type->values[len] != 0) len++;
-    
+
     tuple = PyTuple_New(len);
     if (!tuple) return NULL;
-    
+
     for (i = 0; i < len ; i++) {
         PyTuple_SET_ITEM(tuple, i, PyInt_FromLong(type->values[i]));
     }
 
-        
+
     obj = (typecastObject *)
         typecast_new(PyString_FromString(type->name), tuple, NULL, base);
-    
+
     if (obj) {
         obj->ccast = type->cast;
         obj->pcast = NULL;
@@ -538,7 +543,7 @@ typecast_from_c(typecastObject_initlist *type, PyObject *dict)
 }
 
 PyObject *
-typecast_cast(PyObject *obj, char *str, int len, PyObject *curs)
+typecast_cast(PyObject *obj, char *str, Py_ssize_t len, PyObject *curs)
 {
     PyObject *old, *res = NULL;
     typecastObject *self = (typecastObject *)obj;
@@ -546,7 +551,7 @@ typecast_cast(PyObject *obj, char *str, int len, PyObject *curs)
     /* we don't incref, the caster *can't* die at this point */
     old = ((cursorObject*)curs)->caster;
     ((cursorObject*)curs)->caster = obj;
-    
+
     if (self->ccast) {
         res = self->ccast(str, len, curs);
     }
